# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/builtin/ui/atm_switch_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_atmSwitchConfigPageWidget(object):

    def setupUi(self, atmSwitchConfigPageWidget):
        atmSwitchConfigPageWidget.setObjectName("atmSwitchConfigPageWidget")
        atmSwitchConfigPageWidget.resize(459, 430)
        self.gridLayout_2 = QtWidgets.QGridLayout(atmSwitchConfigPageWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiGeneralGroupBox = QtWidgets.QGroupBox(atmSwitchConfigPageWidget)
        self.uiGeneralGroupBox.setObjectName("uiGeneralGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGeneralGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralGroupBox)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralGroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiVPICheckBox = QtWidgets.QCheckBox(self.uiGeneralGroupBox)
        self.uiVPICheckBox.setObjectName("uiVPICheckBox")
        self.gridLayout.addWidget(self.uiVPICheckBox, 1, 0, 1, 2)
        self.gridLayout_2.addWidget(self.uiGeneralGroupBox, 0, 0, 1, 3)
        self.uiMappingGroupBox = QtWidgets.QGroupBox(atmSwitchConfigPageWidget)
        self.uiMappingGroupBox.setObjectName("uiMappingGroupBox")
        self.vboxlayout = QtWidgets.QVBoxLayout(self.uiMappingGroupBox)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiMappingTreeWidget = QtWidgets.QTreeWidget(self.uiMappingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMappingTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiMappingTreeWidget.setSizePolicy(sizePolicy)
        self.uiMappingTreeWidget.setRootIsDecorated(False)
        self.uiMappingTreeWidget.setObjectName("uiMappingTreeWidget")
        self.vboxlayout.addWidget(self.uiMappingTreeWidget)
        self.gridLayout_2.addWidget(self.uiMappingGroupBox, 0, 3, 3, 1)
        self.uiAddPushButton = QtWidgets.QPushButton(atmSwitchConfigPageWidget)
        self.uiAddPushButton.setObjectName("uiAddPushButton")
        self.gridLayout_2.addWidget(self.uiAddPushButton, 3, 0, 1, 1)
        self.uiDeletePushButton = QtWidgets.QPushButton(atmSwitchConfigPageWidget)
        self.uiDeletePushButton.setEnabled(False)
        self.uiDeletePushButton.setObjectName("uiDeletePushButton")
        self.gridLayout_2.addWidget(self.uiDeletePushButton, 3, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(213, 31, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 4, 2, 1, 2)
        self.uiSourceGroupBox = QtWidgets.QGroupBox(atmSwitchConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSourceGroupBox.sizePolicy().hasHeightForWidth())
        self.uiSourceGroupBox.setSizePolicy(sizePolicy)
        self.uiSourceGroupBox.setObjectName("uiSourceGroupBox")
        self.gridlayout = QtWidgets.QGridLayout(self.uiSourceGroupBox)
        self.gridlayout.setObjectName("gridlayout")
        self.uiSourcePortLabel = QtWidgets.QLabel(self.uiSourceGroupBox)
        self.uiSourcePortLabel.setObjectName("uiSourcePortLabel")
        self.gridlayout.addWidget(self.uiSourcePortLabel, 0, 0, 1, 1)
        self.uiSourcePortSpinBox = QtWidgets.QSpinBox(self.uiSourceGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSourcePortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiSourcePortSpinBox.setSizePolicy(sizePolicy)
        self.uiSourcePortSpinBox.setMinimum(0)
        self.uiSourcePortSpinBox.setMaximum(65535)
        self.uiSourcePortSpinBox.setProperty("value", 1)
        self.uiSourcePortSpinBox.setObjectName("uiSourcePortSpinBox")
        self.gridlayout.addWidget(self.uiSourcePortSpinBox, 0, 1, 1, 1)
        self.uiSourceVPILabel = QtWidgets.QLabel(self.uiSourceGroupBox)
        self.uiSourceVPILabel.setObjectName("uiSourceVPILabel")
        self.gridlayout.addWidget(self.uiSourceVPILabel, 1, 0, 1, 1)
        self.uiSourceVPISpinBox = QtWidgets.QSpinBox(self.uiSourceGroupBox)
        self.uiSourceVPISpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSourceVPISpinBox.sizePolicy().hasHeightForWidth())
        self.uiSourceVPISpinBox.setSizePolicy(sizePolicy)
        self.uiSourceVPISpinBox.setMaximum(65535)
        self.uiSourceVPISpinBox.setProperty("value", 0)
        self.uiSourceVPISpinBox.setObjectName("uiSourceVPISpinBox")
        self.gridlayout.addWidget(self.uiSourceVPISpinBox, 1, 1, 1, 1)
        self.uiSourceVCILabel = QtWidgets.QLabel(self.uiSourceGroupBox)
        self.uiSourceVCILabel.setObjectName("uiSourceVCILabel")
        self.gridlayout.addWidget(self.uiSourceVCILabel, 2, 0, 1, 1)
        self.uiSourceVCISpinBox = QtWidgets.QSpinBox(self.uiSourceGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSourceVCISpinBox.sizePolicy().hasHeightForWidth())
        self.uiSourceVCISpinBox.setSizePolicy(sizePolicy)
        self.uiSourceVCISpinBox.setMaximum(65535)
        self.uiSourceVCISpinBox.setProperty("value", 100)
        self.uiSourceVCISpinBox.setObjectName("uiSourceVCISpinBox")
        self.gridlayout.addWidget(self.uiSourceVCISpinBox, 2, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiSourceGroupBox, 1, 0, 1, 3)
        self.uiDestinationGroupBox = QtWidgets.QGroupBox(atmSwitchConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDestinationGroupBox.sizePolicy().hasHeightForWidth())
        self.uiDestinationGroupBox.setSizePolicy(sizePolicy)
        self.uiDestinationGroupBox.setObjectName("uiDestinationGroupBox")
        self.gridlayout1 = QtWidgets.QGridLayout(self.uiDestinationGroupBox)
        self.gridlayout1.setObjectName("gridlayout1")
        self.uiDestinationPortLabel = QtWidgets.QLabel(self.uiDestinationGroupBox)
        self.uiDestinationPortLabel.setObjectName("uiDestinationPortLabel")
        self.gridlayout1.addWidget(self.uiDestinationPortLabel, 0, 0, 1, 1)
        self.uiDestinationPortSpinBox = QtWidgets.QSpinBox(self.uiDestinationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDestinationPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiDestinationPortSpinBox.setSizePolicy(sizePolicy)
        self.uiDestinationPortSpinBox.setMinimum(0)
        self.uiDestinationPortSpinBox.setMaximum(65535)
        self.uiDestinationPortSpinBox.setProperty("value", 10)
        self.uiDestinationPortSpinBox.setObjectName("uiDestinationPortSpinBox")
        self.gridlayout1.addWidget(self.uiDestinationPortSpinBox, 0, 1, 1, 1)
        self.uiDestinationVPILabel = QtWidgets.QLabel(self.uiDestinationGroupBox)
        self.uiDestinationVPILabel.setObjectName("uiDestinationVPILabel")
        self.gridlayout1.addWidget(self.uiDestinationVPILabel, 1, 0, 1, 1)
        self.uiDestinationVPISpinBox = QtWidgets.QSpinBox(self.uiDestinationGroupBox)
        self.uiDestinationVPISpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDestinationVPISpinBox.sizePolicy().hasHeightForWidth())
        self.uiDestinationVPISpinBox.setSizePolicy(sizePolicy)
        self.uiDestinationVPISpinBox.setMaximum(65535)
        self.uiDestinationVPISpinBox.setProperty("value", 0)
        self.uiDestinationVPISpinBox.setObjectName("uiDestinationVPISpinBox")
        self.gridlayout1.addWidget(self.uiDestinationVPISpinBox, 1, 1, 1, 1)
        self.uiDestinationVCILabel = QtWidgets.QLabel(self.uiDestinationGroupBox)
        self.uiDestinationVCILabel.setObjectName("uiDestinationVCILabel")
        self.gridlayout1.addWidget(self.uiDestinationVCILabel, 2, 0, 1, 1)
        self.uiDestinationVCISpinBox = QtWidgets.QSpinBox(self.uiDestinationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDestinationVCISpinBox.sizePolicy().hasHeightForWidth())
        self.uiDestinationVCISpinBox.setSizePolicy(sizePolicy)
        self.uiDestinationVCISpinBox.setMaximum(65535)
        self.uiDestinationVCISpinBox.setProperty("value", 200)
        self.uiDestinationVCISpinBox.setObjectName("uiDestinationVCISpinBox")
        self.gridlayout1.addWidget(self.uiDestinationVCISpinBox, 2, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiDestinationGroupBox, 2, 0, 1, 3)

        self.retranslateUi(atmSwitchConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(atmSwitchConfigPageWidget)
        atmSwitchConfigPageWidget.setTabOrder(self.uiVPICheckBox, self.uiSourcePortSpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiSourcePortSpinBox, self.uiSourceVPISpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiSourceVPISpinBox, self.uiSourceVCISpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiSourceVCISpinBox, self.uiDestinationPortSpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiDestinationPortSpinBox, self.uiDestinationVPISpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiDestinationVPISpinBox, self.uiDestinationVCISpinBox)
        atmSwitchConfigPageWidget.setTabOrder(self.uiDestinationVCISpinBox, self.uiAddPushButton)
        atmSwitchConfigPageWidget.setTabOrder(self.uiAddPushButton, self.uiDeletePushButton)

    def retranslateUi(self, atmSwitchConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        atmSwitchConfigPageWidget.setWindowTitle(_translate("atmSwitchConfigPageWidget", "ATM Switch"))
        self.uiGeneralGroupBox.setTitle(_translate("atmSwitchConfigPageWidget", "General"))
        self.uiNameLabel.setText(_translate("atmSwitchConfigPageWidget", "Name:"))
        self.uiVPICheckBox.setText(_translate("atmSwitchConfigPageWidget", "Use VPI only (VP tunnel)"))
        self.uiMappingGroupBox.setTitle(_translate("atmSwitchConfigPageWidget", "Mapping"))
        self.uiMappingTreeWidget.headerItem().setText(0, _translate("atmSwitchConfigPageWidget", "Port:VPI:VCI"))
        self.uiMappingTreeWidget.headerItem().setText(1, _translate("atmSwitchConfigPageWidget", "Port:VPI:VCI"))
        self.uiAddPushButton.setText(_translate("atmSwitchConfigPageWidget", "&Add"))
        self.uiDeletePushButton.setText(_translate("atmSwitchConfigPageWidget", "&Delete"))
        self.uiSourceGroupBox.setTitle(_translate("atmSwitchConfigPageWidget", "Source"))
        self.uiSourcePortLabel.setText(_translate("atmSwitchConfigPageWidget", "Port:"))
        self.uiSourceVPILabel.setText(_translate("atmSwitchConfigPageWidget", "VPI:"))
        self.uiSourceVCILabel.setText(_translate("atmSwitchConfigPageWidget", "VCI:"))
        self.uiDestinationGroupBox.setTitle(_translate("atmSwitchConfigPageWidget", "Destination"))
        self.uiDestinationPortLabel.setText(_translate("atmSwitchConfigPageWidget", "Port:"))
        self.uiDestinationVPILabel.setText(_translate("atmSwitchConfigPageWidget", "VPI:"))
        self.uiDestinationVCILabel.setText(_translate("atmSwitchConfigPageWidget", "VCI:"))
